class BMC::SMS::Strategies::Amazon
  attr_reader :region, :access_key_id, :secret_access_key

  # rubocop:disable Layout/LineLength
  def initialize(region: nil, access_key_id: nil, secret_access_key: nil)
    @region            = region            || ENV["SNS_REGION"]            || ENV["AWS_REGION"]
    @access_key_id     = access_key_id     || ENV["SNS_ACCESS_KEY_ID"]     || ENV["AWS_ACCESS_KEY_ID"]
    @secret_access_key = secret_access_key || ENV["SNS_SECRET_ACCESS_KEY"] || ENV["AWS_SECRET_ACCESS_KEY"]
  end
  # rubocop:enable Layout/LineLength

  def client
    @client ||= Aws::SNS::Client.new(
      :region            => region,
      :access_key_id     => access_key_id,
      :secret_access_key => secret_access_key,
    )
  end

  def call(data)
    from = data[:from] || BMC::SMS.default_from

    client.publish(
      :phone_number => data[:to],
      :message => data[:body],
      :message_attributes => {
        "AWS.SNS.SMS.SenderID" => {
          :data_type    => "String",
          :string_value => from,
        },
        "AWS.SNS.SMS.SMSType" => {
          :data_type => "String",
          :string_value => "Transactional",
        },
      },
    )
  end
end
